(* Whlerverhalten 
 *
 * Hier ein Simulationsprogramm, das eine primitive Form des Whlerverhaltens vor einer 
 * Wahl reprsentieren soll. Ausgehend von einer zuflligen Verteilung der Whlermeinung 
 * von 225 Whlern und zwei mglichen Parteien ('.'- und '*'-Partei) geht das Verfahren
 * folgendermaen vor: Ein Whler wird zufllig herausgegriffen, dazu ein zuflliger von 
 * seinen acht Nachbarn. Der Whler bekommt nun die Whlermeinung des Nachbarn auferlegt, 
 * d.h., wenn der Nachbar anderer Meinung war, ndert sich das Wahlverhalten des Whlers,
 * stimmen beide berein, ndert sich nichts. Damit es an den Rndern nicht zu 
 * Schwierigkeiten kommt, ist die Schar der Whler als Torus (aufgeblasener 
 * Autoreifenschlauch) angeordnet. Dargestellt wird er aufgeklappt als Rechteckschema, 
 * bei dem man gegenberliegende Seiten als verbunden ansehen mu. Die Verbindung der 
 * gegenberliegenden Kanten nimmt die Prozedur "torus" mit Hilfe der Koordinaten
 * vor. 
 * 
 * Auch diese Programmidee stammt aus SPEKTRUM der Wissenschaft", Heft 6/85, 
 * das Programm wiederum von mir. 
 *)

(*** Whlersimulation aus Spektrum der Wissenschaft 6/85 ***) 
(*** (c) 1985/89/2001 by Claus Schoenleber, Kiel.        ***) 

PROGRAM election; 

  VAR hours,line,column,neighbor, 
      n_line,n_column,time,minutes, 
      stars,points                  : integer; 
      num                           : integer; 
      elector                       : ARRAY [1..15,1..15] OF char; 
      party                         : string [1]; 

FUNCTION torus (position : integer) : integer; 
  VAR next_elector : integer; 
BEGIN 
  next_elector := position mod 16; 
  IF position = 0 THEN 
    next_elector := 15; 
  IF position = 16 THEN 
    next_elector := 1; 
  torus := next_elector 
END; 

PROCEDURE initialize; 
  VAR x,y : integer; 
BEGIN 
  stars := 0; 
  points := 0; 
  write ('How much hours : '); 
  readln (hours); 
  write ('initializing'); 
  FOR line := 1 TO 15 DO 
  BEGIN 
    write ('.'); 
    FOR column := 1 TO 15 DO 
    BEGIN 
      num := random (2) + 1; 
      CASE num OF 
        1 : BEGIN 
              elector [column,line] := '*'; 
              stars := succ (stars) 
            END; 
        2 : BEGIN 
              elector [column,line] := '.'; 
              points := succ (points) 
            END 
       END 
    END 
  END; 
  clrscr; 
  FOR y := 1 TO 15 DO 
    FOR x := 1 TO 15 DO 
    BEGIN 
      gotoXY (2 * x + 20,y + 3); 
      write (elector [x,y]) 
    END; 
END; 

PROCEDURE screen_output; 
  VAR x,y : integer; 
BEGIN 
  gotoXY (1,1); 
  write ('Time : ',time:4); 
  gotoXY (1,3); 
  writeln ('*-Party : ',stars:3); 
  gotoXY (1,5); 
  writeln ('.-Party : ',points:3); 
  gotoXY (40,3); 
  gotoXY (2 * column + 20,line + 3); 
  write (elector [column,line]) 
END; 

PROCEDURE runtime; 
BEGIN 
  FOR time := 0 TO hours DO 
  BEGIN 
    column := random (15) + 1; 
    line := random (15) + 1; 
    neighbor := random (8) + 1; 
    n_column := column; 
    n_line := line; 
    CASE neighbor OF 
      1 : n_line := torus (line - 1); 
      2 : BEGIN 
            n_column := torus (column + 1); 
            n_line := torus (line - 1) 
          END; 
      3 : n_column := torus (column + 1); 
      4 : BEGIN 
            n_column := torus (column + 1); 
            n_line := torus (line + 1) 
          END; 
      5 : n_line := torus (line + 1); 
      6 : BEGIN 
            n_column := torus (column - 1); 
            n_line := torus (line + 1) 
          END; 
      7 : n_column := torus (column - 1); 
      8 : BEGIN 
            n_column := torus (column - 1); 
            n_line := torus (line - 1) 
          END 
    END; 
    IF (elector [column,line] <> elector [n_column,n_line]) THEN 
    BEGIN 
      party := elector [n_column,n_line]; 
      CASE party [1] OF 
        '*' : BEGIN 
                stars := stars + 1; 
                points := 225 - stars 
              END; 
        '.' : BEGIN 
                points := points + 1; 
                stars := 225 - points 
              END 
      END 
    END; 
    elector [column,line] := elector [n_column,n_line]; 
    screen_output 
  END 
END; 

(*** $MAIN: election ***) 

BEGIN 
  clrscr; 
  randomize; 
  initialize; 
  runtime 
END. 
