(* Sieb des Eratosthenes 
 *
 * Das Berechnen von Primzahlen (Zahlen, die nur durch sich selbst teilbar sind) gehrt 
 * zu den schwierigeren Dingen in der Praktischen Mathematik. Da man keine allgemeine 
 * Formel hat, mu man sich Verfahren ausdenken, die das Berechnen von Primzahlen 
 * wenigstens in ertrglichen Mae zulassen. Ein sehr altes Verfahren ist das Sieb des 
 * Eratosthenes. Es geht zunchst davon aus, da alle Zahlen Primzahlen sind, ausgenommen 
 * die 1, die es laut Definition nicht ist. Man sucht also die nchste Primzahl (das ist 2)
 * und streicht alle ihre Vielfachen. Damit sind die geraden Zahlen schon aus dem Spiel. 
 * Dann sucht man die nchste Primzahl (das ist die nchste im vorigen Durchgang nicht 
 * gestrichene Zahl) und streicht wieder alle ihre Vielfachen. Das macht man bis zu einer 
 * bestimmten Obergrenze. Es ist beweisbar, da es ausreicht, bis zur Quadratwurzel einer 
 * festgelegten obersten Zahl zu gehen, um alle Primzahlen bis zu dieser Obergrenze
 * zu finden. Der Name Sieb rhrt daher, da bei diesem Verfahren alle Primzahlen nach 
 * und nach ausgesiebt werden. 
 *)

PROGRAM eratosthenes; 

  CONST max = 10000; 

  VAR prim : ARRAY [1..max] OF boolean; 
      x,y  : integer; 

BEGIN 
  (*** initialisieren des Zahlenfeldes ***) 
    FOR x := 1 TO max DO 
      prim [x] := true; 
    prim [1] := false; 

  (*** Aussieben der Primzahlen ***) 
    FOR x := 1 TO trunc (sqrt (max)) DO 
    BEGIN 
      IF prim [x] THEN 
      BEGIN 
        y := x + x; 
        REPEAT 
          prim [y] := false; 
          y := y + x; 
        UNTIL (y > max) 
      END; 
    END; 

  (*** Ausgabe ***) 
    FOR x := 1 TO max DO 
      IF prim [x] THEN 
        write (x:5); 
END. 